@@ -19,7 +19,9 @@ environment variable
 '''
 
 
+from subprocess import check_output
 import os
+import re
 
 try:
     from . import changelogparser
@@ -27,26 +29,46 @@ except:
     import changelogparser
 
 
+def git_get_info():
+    '''Returns a tuple of three elements:
+
+     - the name of the current commit's closest tag
+     - the distance in commits to that tag (or 0 if we are on the tag)
+     - the short SHA of the current commit'''
+    out = check_output(['git', 'describe', '--tags', '--long']).decode('ascii')
+    m = re.search('(.*)-([0-9]+)-g(.*)', out)
+    if not m:
+        raise ValueError("Could not parse git command's output: {}".format(out))
+    return (m.group(1), int(m.group(2)), m.group(3))
+
+def get_version_from_tag(tag):
+    return tag.lstrip('v')
+
 def get_changelog_version(filename):
     with open(filename, 'r') as clfile:
         changelog = changelogparser.ChangeLogRuleVisitor().parse(clfile.read())
         return changelogparser.version_string(
             changelogparser.changelog_version(changelog))
 
+def get_version_using_git():
+    tag, tag_distance, sha = git_get_info()
+    if tag_distance == 0:
+        return get_version_from_tag(tag)
+    return "{}+{}".format(get_changelog_version("CHANGELOG.md"), sha)
+
+def get_version():
+    if "CI_COMMIT_TAG" in os.environ:
+        return get_version_from_tag(os.environ["CI_COMMIT_TAG"])
+
+    if "CI_COMMIT_SHORT_SHA" not in os.environ:
+        return get_version_using_git()
+
+    return "{}+{}".format(get_changelog_version("CHANGELOG.md"),
+                          os.environ["CI_COMMIT_SHORT_SHA"])
 
 def main():
     with open("version.txt", "w") as vfile:
-        if "CI_COMMIT_TAG" in os.environ:
-            tag = os.environ["CI_COMMIT_TAG"]
-            vfile.write(tag.lstrip('v') + "\n")
-        else:
-            if "CI_COMMIT_SHORT_SHA" not in os.environ:
-                raise ValueError(
-                    "CI_COMMIT_SHORT_SHA environment variable missing")
-            version = "{}+{}".format(get_changelog_version("CHANGELOG.md"),
-                                     os.environ["CI_COMMIT_SHORT_SHA"])
-            vfile.write(version + "\n")
-
+        vfile.write(get_version() + '\n')
 
 if __name__ == "__main__":
     main()
